from . import TextStatistics as stats

def wfFormIndex(text, lang):
    """Returns the Formality Score of a text.
    
    This function returns a value based on the weight that each kind of word
    (adjective, noun, pronoun, etc.) has within the text.
    
    """

    diffWords = stats.wordClass(text, lang)
    tokens = stats.tokenizar(text, lang)
    wordsNumber = len(tokens)
    
    formalityIndex = ((diffWords[3] + diffWords[1] + diffWords[5] + diffWords[6] - diffWords[7] - diffWords[2]
                        - diffWords[4] - diffWords[8])/wordsNumber*100  +100)/2
    
    return formalityIndex
    

def adfScore(text, lang):
    """Returns the Adjective Score of a text.
    
    This function returns the Adjective Score of a text, which is a formality
    measure of a text.
    
    """
    diffWords = stats.wordClass(text, lang)
    wordsNumber = len(stats.tokenizar(text, lang))
    
    adf = diffWords[1]/wordsNumber * 100
    
    return adf


#https://github.com/mabodo/sibilizador/blob/master/Silabizator.ipynb
class char():
    def __init__(self):
        pass

class char_line():
    def __init__(self, word):
        self.word = word
        self.char_line = [(char, self.char_type(char)) for char in word]
        self.type_line = ''.join(chartype for char, chartype in self.char_line)
        
    def char_type(self, char):
        if char in set(['a', 'á', 'e', 'é','o', 'ó', 'í', 'ú']):
            return 'V' #strong vowel
        if char in set(['i', 'u']):
            return 'v' #week vowel
        if char=='x':
            return 'x'
        if char=='s':
            return 's'
        else:
            return 'c'
            
    def find(self, finder):
        return self.type_line.find(finder)
        
    def split(self, pos, where):
        return char_line(self.word[0:pos+where]), char_line(self.word[pos+where:])
    
    def split_by(self, finder, where):
        split_point = self.find(finder)
        if split_point!=-1:
            chl1, chl2 = self.split(split_point, where)
            return chl1, chl2
        return self, False
     
    def __str__(self):
        return '<'+self.word+':'+self.type_line+'>'
    
    def __repr__(self):
        return '<'+repr(self.word)+':'+self.type_line+'>'

class spanishSilabizer():
    def __init__(self):
        self.grammar = []
        
    def split(self, chars):
        rules  = [('VV',1), ('cccc',2), ('xcc',1), ('ccx',2), ('csc',2), ('xc',1), ('cc',1), ('vcc',2), ('Vcc',2), ('sc',1), ('cs',1),('Vc',1), ('vc',1), ('Vs',1), ('vs',1), ('vxv',1), ('VxV',1), ('vxV',1), ('Vxv',1)]
        for split_rule, where in rules:
            first, second = chars.split_by(split_rule,where)
            if second:
                if first.type_line in set(['c','s','x','cs']) or second.type_line in set(['c','s','x','cs']):
                    #print 'skip1', first.word, second.word, split_rule, chars.type_line
                    continue
                if first.type_line[-1]=='c' and second.word[0] in set(['l','r']):
                    continue
                if first.word[-1]=='l' and second.word[-1]=='l':
                    continue
                if first.word[-1]=='r' and second.word[-1]=='r':
                    continue
                if first.word[-1]=='c' and second.word[-1]=='h':
                    continue
                return self.split(first)+self.split(second)
        return [chars]
        
    def __call__(self, word):
        return self.split(char_line(word))

#https://stackoverflow.com/questions/46759492/syllable-count-in-python
class englishSilabizer():
    
    def __init__(self):
        pass
    
    def syllable_count(self, word):
        word = word.lower()
        count = 0
        vowels = "aeiouy"
        if word[0] in vowels:
            count += 1
        for index in range(1, len(word)):
            if word[index] in vowels and word[index - 1] not in vowels:
                count += 1
        if word.endswith("e"):
            count -= 1
        if count == 0:
            count += 1
        return count
    
    def __call__(self, word):
        return self.syllable_count(word)
    


from . import TextStatistics as stats
from . import RIAlgorithms as ri
from . import VocabularyRichness as vr
from . import formality as form
from . import coherenceMetrics as coh

class StyleMetrics():

	def _setReadabilityIndex(self, text, lang):


		if lang == "en":
			readabilityIndexes = {'ARI': ri.ARI(text, lang), 
							'FleschRI': ri.fleschRI(text, lang),
							'FogCount': ri.fogCount(text, lang)
							}
		else:
			readabilityIndexes = {'INFLESZ': ri.INFLESZ(text, lang),
							'Mu': ri.muReadability(text, lang)
							}

		return readabilityIndexes

	def _setTextStatistics(self, text, lang):

		diffWords = stats.differentWords(text, lang)
		commonUncommonWords = stats.commonUncommonWords2(text, lang)
		wordsClasses = stats.wordClass(text, lang)

		statistics = {'sentencesCount': stats.sentencesCount(text, lang)[0],
							'charPerSent': stats.charactersPerSentence(text, lang),
							'charPerWord': stats.charactersPerWord(text, lang),
							'wordsPerSent': stats.wordsPerSentence(text, lang),
							'diffWords': diffWords[0],
							'conjunctions': wordsClasses[0],
							'adjectives': wordsClasses[1],
							'verbs': wordsClasses[2],
							'nouns': wordsClasses[3],
							'adverbs': wordsClasses[4],
							'prepositions': wordsClasses[5],
							'determiners': wordsClasses[6],
							'pronouns': wordsClasses[7],
							'interjections': wordsClasses[8],
							'shortWords': stats.shortWords(text, lang),
							'commonWords': commonUncommonWords[0],
							'uncommonWords': commonUncommonWords[1],
							}

		return statistics

	def __init__(self, text, lang, window, limit, coherence, path):
		if coherence:
			self._cIndex = coh.coherenceMeasure(text, lang, path)
		else:
			self._cIndex = 0
		self._fIndex = form.wfFormIndex(text, lang)
		self._adjScore = form.adfScore(text, lang)
		self._vIndex = {'TTR': vr.TTR(text, lang),
		 				'Others': vr.vbRichFunction(text, lang),
		 				'MSTTR': vr.MSTTR(text, lang), 
		 				'MATTR': vr.MATTR(text, window, lang),
						'MTLD': vr.MTLD(text, limit, lang),
						'HDD': vr.HDD(text, lang)
						}
		self._rIndex = self._setReadabilityIndex(text, lang)
		self._textStats = self._setTextStatistics(text, lang)
		
		
		


	def getCoherenceIndex(self):
		return self._cIndex

	def getFormalityIndex(self):
		return self._fIndex

	def getAdjScore(self):
		return self._adjScore

	def getReadabilityIndex(self, index = ''):

		result = ''

		if index in self._rIndex:
			result = self._rIndex[index]
		elif index == '':
			result = self._rIndex
		else:
			result = 'ERROR: Bad Index'

		return result

	def getVocabularyIndex(self, index = ''):

		result = ''

		if index in self._vIndex:
			result = self._vIndex[index]
		elif index == '':
			result = self._vIndex
		else:
			result = 'ERROR: Bad Index'

		return result

	def getTextStatistics(self, index = ''):

		result = ''

		if index in self._textStats:
			result = self._textStats[index]
		elif index == '':
			result = self._textStats
		else:
			result = 'ERROR: Bad Index'

		return result



from gensim.models.keyedvectors import KeyedVectors
from gsitk.features.word2vec import Word2VecFeatures
import numpy
from . import TextStatistics as stats



def coherenceMeasure(text, lang, path):
    """Returns the cocherence of a text.
    
    This function measures the coherence of a text.
    First, a Word Embeddings model is loaded. Then, the text is tokenized in sentences.
    The sentences are tokenized too. Finally, the cosine similarity of the sentences
    is measured and the final result is the average of these measures.

    The available Word Embeddings formats are .bin for English texts and .txt for Spanish texts.
    
    """

    if lang == "en":
        w2v_extractor = Word2VecFeatures(w2v_model_path=path, w2v_format='google_bin', convolution=[1,0,0])
    elif lang == "es":
         w2v_extractor = Word2VecFeatures(w2v_model_path=path, w2v_format='google_txt', convolution=[1,0,0])

    sentences = stats.sentencesCount(text, lang)

    sentList = []

    for sent in range(sentences[0]):
        sentList.append(stats.tokenizar(sentences[1][sent], lang))

    transOpt = w2v_extractor.transform(sentList)

    transOpt[~numpy.isnan(transOpt).any(axis=1)]
    transOpt = transOpt[~numpy.all(transOpt == 0, axis=1)]

    sum = 0
    for sent in range(len(transOpt)-1):
        sims = w2v_extractor.model.cosine_similarities(transOpt[sent], transOpt)
        sum += sims[sent + 1]

    return sum/(len(transOpt) - 1) * 100


import nltk
nltk.download('punkt')
nltk.download("popular")
from nltk.corpus import stopwords
from nltk.stem.wordnet import WordNetLemmatizer
from nltk.tag import StanfordPOSTagger
from nltk import FreqDist
import wordfreq
import contractions


def tokenizar(text, lang):
    """Splits in words a text.
    
    This function removes undesirable characters and extends English contractions.
    Then it appends each word to a list.
    Returns that list of words.
    
    """
    
    procText = text
    
    chars1 = "`´"
    chars2 = "-–¿¡/\\|"
    
    for char in chars1:
        procText = procText.replace(char, "'")
   
    if lang == "en":
        procText = contractions.fix(procText)
    
    for char in chars2:
        procText = procText.replace(char, ' ')
        
    procText = procText.replace(" \'", " ")
    
    tokens = nltk.word_tokenize(procText)
    tokens = ([token for token in tokens
               if any(c.isalpha() for c in token)])
    
    tokensOpt = []
    for token in tokens:
        if token[0] == "'":
            token = token[1:len(token)]
            
        elif token[len(token)-1] == ".":
            token = token[0:len(token)-1]
        if token != 's':
            tokensOpt.append(token)
    return tokensOpt

def sentencesCount(text, lang):
    """Calculates the number of sentences in the text.
    
    It tokenizes the text in different sentences thanks to a pre-trained Punkt tokenizer
    for the English language and for the Spanish language.
    Then, it counts how many sentences there are.
    Returns the number of sentences and a list with the text sentences.
    
    """
    procText = text
    
    chars1 = "`´"
    chars2 = "-–¿¡/\\"
    chars3 = "\n"
    chars4 = '  '
    
    for char in chars1:
        procText = procText.replace(char, "'")
   
    if lang == "en":
        procText = contractions.fix(procText)
         #The NLTK data package includes a pre-trained Punkt tokenizer for English.
        sent_detector = nltk.data.load('tokenizers/punkt/english.pickle')
        
    elif lang == "es":
        sent_detector = nltk.data.load('tokenizers/punkt/spanish.pickle')
        
    for char in chars2:
        procText = procText.replace(char, ' ')
        
    procText = procText.replace(chars3, ' ')
    procText = procText.replace(chars4, ' ')  
    
        
    sentences = sent_detector.tokenize(procText.strip())
    sentNumber = len(sentences)
    
    for sent in sentences:
        if sent.find('\n\n') != -1:
            sentNumber +=1

    return sentNumber, sentences

def charactersPerSentence(text, lang):
    """Calculates an average of the number of characters per sentence.
    
    The function returns an average of the number of characters per sentence.
    
    """
    sentencesNumber = sentencesCount(text,lang)[0]
    tokens = tokenizar(text, lang)

    charPerSent = 0
    
    for token in tokens:
        charPerSent += len(token)
        
    charPerSent /= sentencesNumber
    
    return charPerSent

def charactersPerWord(text, lang):
    """Calculates an average of the number of characters per word.
    
    The function returns an average of the number of characters per word.
    
    """
    tokens = tokenizar(text, lang)
    
    wordsNumber = len(tokens)
    
    charPerWord = 0
    
    for token in tokens:
        charPerWord += len(token)
        
    charPerWord /= wordsNumber
    
    return charPerWord


def differentWords(text, lang):
    """Identifies the different words within the text.
    
    Returns the number of different words and a list with them.
    The function replaces uppercase with lowercase.
    
    """
    
    diffWordsList = []
    tokens = tokenizar(text, lang)
    tokensLower = []
    for token in tokens:
        tokensLower.append(token.lower())
    tokensLower.sort()

    i = 0
    for token in tokensLower:
        if i+1 < len(tokensLower):
            if token != tokensLower[i + 1]:
                diffWordsList.append(token)
        else:
            diffWordsList.append(token)
        i += 1
    
    return len(diffWordsList), diffWordsList

def wordClass(text, lang):
    """It searchs the different types of words in the text
    
    It tokenizes the text in tagged words. Then, and thanks to the tags,
    it classifies the words by class and save the words in different lists.
    It returns the number of conjunctions, adjectives, verbs, nouns, adverbs, prepositions,
    determiners, pronouns and interjections..
    
    """

    tokens = tokenizar(text, lang)

    
    nouns = []
    verbs = []
    adjectives = []
    adverbs = []
    conjunctions = []
    prepositions = []
    determiners = []
    pronouns = []
    interjections =[]
    
    if lang == "en":
        
        taggedTokens = nltk.pos_tag(tokens)
        
        for tupla in taggedTokens:
            if tupla[1] == "CC":
                conjunctions.append(tupla[0])
            elif tupla[1][0] == "J":
                adjectives.append(tupla[0])
            elif tupla[1][0] == "V":
                verbs.append(tupla[0])
            elif tupla[1][0] == "N":
                nouns.append(tupla[0])
            elif tupla[1][0] == "R" or tupla[1] == "WRB":
                adverbs.append(tupla[0])
            elif tupla[1] == "TO" or tupla[1] == "IN":
                prepositions.append(tupla[0])
            elif tupla[1] == "WDT" or tupla[1] == "PDT" or tupla[1] == "DT":
                determiners.append(tupla[0])
            elif tupla[1] == "PRP" or tupla[1] == "PRP$" or tupla[1] == "WP" or tupla[1] == "WP$":
                pronouns.append(tupla[0])
            elif tupla[1] == "UH":
                interjections.append(tupla[0])
            
        result = [len(conjunctions), len(adjectives), len(verbs), len(nouns), len(adverbs), len(prepositions),
                  len(determiners), len(pronouns), len(interjections)]
            
            
    elif lang == "es":
        
        tagger = r"stylomepy/stanfordPosTagger/models/spanish.tagger"
        jar = r"stylomepy/stanfordPosTagger/stanford-postagger.jar"
        taggedTokens = StanfordPOSTagger(tagger,jar).tag(tokens)

        for tupla in taggedTokens:
            if tupla[1][0] == "c":
                conjunctions.append(tupla[0])
            elif tupla[1][0] == "a":
                adjectives.append(tupla[0])
            elif tupla[1][0] == "v":
                verbs.append(tupla[0])
            elif tupla[1][0] == "n":
                nouns.append(tupla[0])
            elif tupla[1][0] == "r":
                adverbs.append(tupla[0])
            elif tupla[1][0] == "s":
                prepositions.append(tupla[0])
            elif tupla[1][0] == "d":
                determiners.append(tupla[0])
            elif tupla[1][0] == "p":
                pronouns.append(tupla[0])
            elif tupla[1][0] == "i":
                interjections.append(tupla[0])
        
        result = [len(conjunctions), len(adjectives), len(verbs), len(nouns), len(adverbs), len(prepositions),
                  len(determiners), len(pronouns), len(interjections)]
    
    
    return result


def wordsPerSentence(text, lang):
    """Calculates an average of the words per sentence.
    
    Return the average of words per sentence.
    
    """    
    sentences = sentencesCount(text, lang)[1]
    
    tokens = tokenizar(text, lang)

    wordsPerSent = len(tokens)/len(sentences)
    
    return wordsPerSent

def shortWords(text, lang):
    """It counts the number of short words within the text.
    
    It returns the number of words with 3 characters or less (Short Words).
    
    """
    tokens = tokenizar(text, lang)

    shortWords = 0
    for token in tokens:
        if len(token) <= 3:
            shortWords += 1
            
    return shortWords

def commonUncommonWords1(text, lang):
    """It counts the common and uncommon words in the text.
    
    Based in the comparation of the words in text with an english dictionary.
    If the word exists in the dictionary, we add 1 to common words. If doesn't,
    we will add 1 to uncommon words.
    It returns the number of common and uncommon words.
    It is only value for English texts.
    
    """
    diffWords = differentWords(text, lang)
    wnLem = WordNetLemmatizer()
    tokensText = ([token.lower() for token in diffWords[1]
                    if any(c.isalpha() for c in token)])
    englishVocab = ([token.lower() for token in nltk.corpus.words.words()
                      if any(c.isalpha() for c in token)])
    
    commonWords = 0
    uncommonWords = 0
    #tags=['a','n','v','r']
    
    def find(token):
        for englishWord in englishVocab:
            if token == englishWord:
                return True
        return False

    for token in tokensText:
        
        if find(token):
            commonWords += 1
        elif find(wnLem.lemmatize(token, pos='a')):
            commonWords += 1
        elif find(wnLem.lemmatize(token, pos='n')):
            commonWords += 1
        elif find(wnLem.lemmatize(token, pos='v')):
            commonWords += 1
        elif find(wnLem.lemmatize(token, pos='r')):
            commonWords += 1
        else:
            uncommonWords += 1
             
    return commonWords, uncommonWords

#Zipf Law
def commonUncommonWords2(text, lang):
    """ Analyzes a text searching for common and uncommon words.
    
    It uses wordfreq library for searching common and uncommon words
    in English or Spanish languages.
    
    Returns the number of common and uncommon words.
    
    """
    diffWords = differentWords(text, lang)[1]
    
    commonWords = 0
    uncommonWords = 0
    
    for token in diffWords:
        if wordfreq.zipf_frequency(token, lang) > 2:
            commonWords += 1
        else:
            uncommonWords += 1
            
    return commonWords, uncommonWords

def freqWords(text, lang):
    """Calculates the frequency of each word within a text
    
    Returns de most common words.
    
    """
    tokens = tokenizar(text, lang)
    tokensLower = []
    for token in tokens:
        tokensLower.append(token.lower())
    fdist = FreqDist(tokensLower)
    return fdist.most_common()

def removeStopWords(text, lang):
    """Removes the stop words in English or Spanish languages
    
    """
    if lang == "en":
        stopWords = set(stopwords.words("english"))
    elif lang == "es":
        stopWords = set(stopwords.words("spanish"))
    words = tokenizar(text, lang)
    wordsFiltered = []

    for w in words:
        if w not in stopWords:
            wordsFiltered.append(w)
 
    return wordsFiltered, len(wordsFiltered)



import setuptools

def parse_requirements(filename):
    """ load requirements from a pip requirements file """
    with open(filename, 'r') as f:
        lineiter = list(line.strip() for line in f)
    return [line for line in lineiter if line and not line.startswith("#")]


install_reqs = parse_requirements("requirements.txt")

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.md') as f:
    readme = f.read()

setup(
  name = 'stylomepy',
  data_files = [("", ["LICENSE.txt", "requirements.txt"], )],
  install_requires=install_reqs,
  packages=setuptools.find_packages(),
  include_package_data=True,
  version = '0.2',
  description = 'A Python library for measuring the style.',
  long_description='Stylomepy is a library that allows you to measure the style of Spanish and English texts.',
  author = 'Álvaro de Pablo',
  author_email = 'alvarodepablomarsal@gmail.com',
  url = 'https://github.com/alvarodepablo/stylomepy',
  keywords = ['style', 'measure', 'vocabulary', 'coherence', 'formality', 'readability', 'text'],
  classifiers = [
  'Development Status :: 3 - Alpha',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3',
  ],
  python_requires='>=3',
  )


from __future__ import unicode_literals
import nltk
from . import TextStatistics as stats
import math
from .Silabizator import englishSilabizer, spanishSilabizer

def ARI(text, lang):
    """Calculates the Automated Readability Index of an English text.
    
    The function returns the two available ARI's metrics of the library.
    The first is the clasical ARI and the second is the optimized ARI.
    
    """

    tokens = stats.tokenizar(text, lang)
    sentenceNumber = stats.sentencesCount(text,lang)[0]
    wordsNumber = len(tokens)
    
    charNumber = 0
    for token in tokens:
        charNumber += len(token)


    avgCharPerWord = charNumber/wordsNumber
    avgWordsPerSentence = wordsNumber/sentenceNumber


    oldReadIndex = math.ceil(4.71*(avgCharPerWord) + 0.5*(avgWordsPerSentence) - 21.43)

    newReadIndex = math.ceil(5.84*(avgCharPerWord) + 0.37*(avgWordsPerSentence) - 26.01)
            
    return oldReadIndex, newReadIndex


def fleschRI(text, lang):
    """Calculates the Flesch Readability Index of an English text.
    
    This function returns the two available Flesch's metrics of the library.
    The first is the Flesch Reading Ease Index and the second is the
    Flesch-Kincaid Grade Level Index.
    
    """
    
    tokens = stats.tokenizar(text, lang)
    sentenceNumber = stats.sentencesCount(text,lang)[0]
    wordsNumber = len(tokens)

    syllablesNumber = 0
    s = englishSilabizer()
    for token in tokens:
        syllablesNumber += s(token)    

    avgSyllPerWord = syllablesNumber/wordsNumber
    avgWordsPerSentence = wordsNumber/sentenceNumber

    fleschReadingEase = 206.835 - 1.015*(avgWordsPerSentence) - 84.6*(avgSyllPerWord)
    fleschKincaidGradeLevel = 0.39*(avgWordsPerSentence) + 11.8*(avgSyllPerWord) - 15.59
    
    return fleschReadingEase, fleschKincaidGradeLevel

def fogCount(text, lang):
    """Calculates the Fog Count Index of an English text.
    
    The function returns the two available Fog Count's metrics of the library.
    The first is the clasical Fog Count and the second is the optimized Fog Count.
    
    """

    tokens = stats.tokenizar(text, lang)
    sentenceNumber = stats.sentencesCount(text,lang)[0]
    wordsNumber = len(tokens)
    oldFogCount = 0
    newFogCount = 0

    easyWords = []
    hardWords = []
    s = englishSilabizer()
    for token in tokens:
        if s(token) > 2:
            hardWords.append(token)
        else:
            easyWords.append(token)


    avgFogCount = (len(easyWords) + 3 * len(hardWords))/sentenceNumber

    if avgFogCount >= 20:
        oldFogCount = avgFogCount/2
    else:
        oldFogCount = (avgFogCount - 2)/2
    
    newFogCount = (avgFogCount - 3)/2
                
    return oldFogCount, newFogCount

def INFLESZ(text, lang):
    """Calculates the INFLESZ Index of a Spanish text.
    
    The function returns the INFLESZ Readability Index value of a
    text written in Spanish.
    
    """

    tokens = stats.tokenizar(text, lang)
    sentenceNumber = stats.sentencesCount(text,lang)[0]
    wordsNumber = len(tokens)

    s = spanishSilabizer()
    syllablesNumber = 0
    for token in tokens:
        syllablesNumber += len(s(token))
    
    inflesz = 206.835 - ((62.3*syllablesNumber)/wordsNumber)-(wordsNumber/sentenceNumber)
    
    return inflesz

def muReadability(text, lang):
    """Calculates the mu Readability Index of a Spanish text.
    
    The function returns the mu Readbility Index of a text written
    in Spanish.
    
    """

    tokens = stats.tokenizar(text, lang)
    wordsNumber = len(tokens)
    charsPerWord = stats.charactersPerWord(text, lang)
    
    variance = 0
    for token in tokens:
        variance += ((len(token)-charsPerWord)**2)
        
    variance /= wordsNumber
    
    mu = ((wordsNumber/(wordsNumber - 1)) * (charsPerWord/variance)) * 100
        
    return mu   


from . import TextStatistics as stats
import math
from scipy.stats import hypergeom



def TTR(text, lang):
    tokens = stats.tokenizar(text, lang)
    wordsNumber = len(tokens)
    diffWords = stats.differentWords(text, lang)[0]
    #TTR Type/Token ratio
    TTR = (diffWords/wordsNumber)
    return TTR

def vbRichFunction(text, lang):
    
    tokens = stats.tokenizar(text, lang)
    wordsNumber = len(tokens)
    diffWords = stats.differentWords(text, lang)[0]

    #Guiraud index
    guiraud = (diffWords/math.sqrt(wordsNumber))
    #Index Herdan
    herdan = (math.log10(diffWords)/math.log10(wordsNumber))
    #Index Uber
    uber = (math.pow((math.log10(wordsNumber)),2)/(math.log10(wordsNumber)-math.log10(diffWords)))
    #Mass
    mass = (math.log10(wordsNumber) - math.log10(diffWords))/(math.pow(math.log10(wordsNumber),2))
    
    
    return guiraud, herdan, uber, mass


def MSTTR(text, lang):
    
    tokens = stats.tokenizar(text, lang)
    wordsNumber = len(tokens)
    if wordsNumber < 100:
        return "The text length is too short to apply MSTTR"
    MSTTR = 0
    i = 0
    auxList = []
    segmentsNumber = 0
    for token in tokens:
        if len(auxList) < 100:
            auxList.append(token)              
            i += 1
        else:
            MSTTR += TTR(' '.join(auxList), lang)
            auxList = []
            segmentsNumber += 1
            if wordsNumber - i >= 100:
                continue
            else:
                break
    for token in reversed(tokens):
    	if len(auxList) < wordsNumber - i:
    		auxList.append(token)
    	else:
    		break
    MSTTR += TTR(' '.join(auxList), lang)
    segmentsNumber += 1
                
    return MSTTR/segmentsNumber * 100, segmentsNumber

def MATTR(text, window, lang):
    tokens = stats.tokenizar(text, lang)
    wordsNumber = len(tokens)
    if window > wordsNumber:
        return "The text is too short to apply this window"
    MATTR = 0
    auxList = []
    j = 0
    while j + window <= wordsNumber:
        for i in range(j, window + j):
            auxList.append(tokens[i])
        j += 1
        MATTR += TTR(' '.join(auxList), lang)
        auxList = []
    return MATTR/(j) *100

def MTLD(text, limit, lang):
    tokens = stats.tokenizar(text, lang)
    wordsNumber = len(tokens)
    def MTLDimplementation(usedTokens, limit):
        nSegm = 0
        segment = []

        for i in range(0, wordsNumber):
            segment.append(usedTokens[i])
            if TTR(' '.join(segment), lang) < limit:
                nSegm += 1
                segment = []
            elif i == wordsNumber - 1 and TTR(' '.join(segment), lang) > limit:
                rest = 1 - TTR(' '.join(segment), lang)
                nSegm += rest/(1 - limit)
        return wordsNumber/nSegm
    
    MTLD1 = MTLDimplementation(tokens, limit)
    MTLD2 = MTLDimplementation(tokens[::-1], limit)
    
    result = (MTLD1 + MTLD2)/2

    if result > 100:
        return 100
    else:
        return result

def HDD(text, lang):
    tokens = stats.tokenizar(text, lang)
    wordsNumber = len(tokens)
    freqWords = stats.freqWords(text, lang)
    probSum = 0
    k = 1
    lista = []
    for i in range(0, len(freqWords)):
        [N, d, n] = [wordsNumber, freqWords[i][1], 42]
        probType = hypergeom.pmf(k, N, d, n)
        probSum += probType
        lista.append(probType)  
        
    return probSum/42 *100